@@ -463,23 +463,25 @@ CAMLprim value ml_z_of_nativeint(value v)
 CAMLprim value ml_z_of_int32(value v)
 {
   int32_t x;
-  value r;
   Z_MARK_OP;
   x = Int32_val(v);
-#if Z_USE_NATINT
-#ifdef ARCH_SIXTYFOUR
+#if Z_USE_NATINT && defined(ARCH_SIXTYFOUR)
   return Val_long(x);
 #else
+#if Z_USE_NATINT
   if (Z_FITS_INT(x)) return Val_long(x);
 #endif
+  {
+    value r;
+    Z_MARK_SLOW;
+    r = ml_z_alloc(1);
+    if (x > 0) { Z_HEAD(r) = 1; Z_LIMB(r)[0] = x; }
+    else if (x < 0) { Z_HEAD(r) = 1 | Z_SIGN_MASK; Z_LIMB(r)[0] = -(mp_limb_t)x; }
+    else Z_HEAD(r) = 0;
+    Z_CHECK(r);
+    return r;
+  }
 #endif
-  Z_MARK_SLOW;
-  r = ml_z_alloc(1);
-  if (x > 0) { Z_HEAD(r) = 1; Z_LIMB(r)[0] = x; }
-  else if (x < 0) { Z_HEAD(r) = 1 | Z_SIGN_MASK; Z_LIMB(r)[0] = -(mp_limb_t)x; }
-  else Z_HEAD(r) = 0;
-  Z_CHECK(r);
-  return r;
 }
 
 CAMLprim value ml_z_of_int64(value v)
@@ -1738,7 +1740,6 @@ CAMLprim value ml_z_gcd(value arg1, value arg2)
     /* fast path */
     intnat a1 = Long_val(arg1);
     intnat a2 = Long_val(arg2);
-    if (!a1 || !a2) ml_z_raise_divide_by_zero();
     if (a1 < 0) a1 = -a1;
     if (a2 < 0) a2 = -a2;
     if (a1 < a2) { intnat t = a1; a1 = a2; a2 = t; }
@@ -1757,52 +1758,55 @@ CAMLprim value ml_z_gcd(value arg1, value arg2)
     mp_size_t sz, pos1, pos2, limb1, limb2, bit1, bit2, pos, limb, bit, i;
     Z_DECL(arg1); Z_DECL(arg2);
     Z_ARG(arg1);  Z_ARG(arg2);
-    if (!size_arg1 || !size_arg2) ml_z_raise_divide_by_zero();
-    /* copy args to tmp storage & remove lower 0 bits */
-    pos1 = mpn_scan1(ptr_arg1, 0);
-    pos2 = mpn_scan1(ptr_arg2, 0);
-    limb1 = pos1 / Z_LIMB_BITS;
-    limb2 = pos2 / Z_LIMB_BITS;
-    bit1 = pos1 % Z_LIMB_BITS;
-    bit2 = pos2 % Z_LIMB_BITS;
-    size_arg1 -= limb1;
-    size_arg2 -= limb2;
-    tmp1 = ml_z_alloc(size_arg1 + 1);
-    tmp2 = ml_z_alloc(size_arg2 + 1);
-    Z_REFRESH(arg1);
-    Z_REFRESH(arg2);
-    if (bit1) {
-      mpn_rshift(Z_LIMB(tmp1), ptr_arg1 + limb1, size_arg1, bit1);
-      if (!Z_LIMB(tmp1)[size_arg1-1]) size_arg1--;
-    }
-    else ml_z_cpy_limb(Z_LIMB(tmp1), ptr_arg1 + limb1, size_arg1);
-    if (bit2) { 
-      mpn_rshift(Z_LIMB(tmp2), ptr_arg2 + limb2, size_arg2, bit2);
-      if (!Z_LIMB(tmp2)[size_arg2-1]) size_arg2--;
-    }
-    else ml_z_cpy_limb(Z_LIMB(tmp2), ptr_arg2 + limb2, size_arg2);
-    /* compute gcd of 2^pos1 & 2^pos2 */
-    pos = (pos1 <= pos2) ? pos1 : pos2;
-    limb = pos / Z_LIMB_BITS;
-    bit = pos % Z_LIMB_BITS;
-    /* compute gcd of arg1 & arg2 without lower 0 bits */
-    /* second argument must have less bits than first  */
-    if ((size_arg1 > size_arg2) ||
-        ((size_arg1 == size_arg2) && 
-         (Z_LIMB(tmp1)[size_arg1 - 1] >= Z_LIMB(tmp2)[size_arg1 - 1]))) {
-      r = ml_z_alloc(size_arg2 + limb + 1);
-      sz = mpn_gcd(Z_LIMB(r) + limb, Z_LIMB(tmp1), size_arg1, Z_LIMB(tmp2), size_arg2);
-    }
+    if (!size_arg1) r = arg2;
+    else if (!size_arg2) r = arg1;
     else {
-      r = ml_z_alloc(size_arg1 + limb + 1);
-      sz = mpn_gcd(Z_LIMB(r) + limb, Z_LIMB(tmp2), size_arg2, Z_LIMB(tmp1), size_arg1);
-    } 
-    /* glue the two results */
-    for (i = 0; i < limb; i++) 
-      Z_LIMB(r)[i] = 0;
-    Z_LIMB(r)[sz + limb] = 0;
-    if (bit) mpn_lshift(Z_LIMB(r) + limb, Z_LIMB(r) + limb, sz + 1, bit);
-    r = ml_z_reduce(r, limb + sz + 1, 0);
+      /* copy args to tmp storage & remove lower 0 bits */
+      pos1 = mpn_scan1(ptr_arg1, 0);
+      pos2 = mpn_scan1(ptr_arg2, 0);
+      limb1 = pos1 / Z_LIMB_BITS;
+      limb2 = pos2 / Z_LIMB_BITS;
+      bit1 = pos1 % Z_LIMB_BITS;
+      bit2 = pos2 % Z_LIMB_BITS;
+      size_arg1 -= limb1;
+      size_arg2 -= limb2;
+      tmp1 = ml_z_alloc(size_arg1 + 1);
+      tmp2 = ml_z_alloc(size_arg2 + 1);
+      Z_REFRESH(arg1);
+      Z_REFRESH(arg2);
+      if (bit1) {
+        mpn_rshift(Z_LIMB(tmp1), ptr_arg1 + limb1, size_arg1, bit1);
+        if (!Z_LIMB(tmp1)[size_arg1-1]) size_arg1--;
+      }
+      else ml_z_cpy_limb(Z_LIMB(tmp1), ptr_arg1 + limb1, size_arg1);
+      if (bit2) { 
+        mpn_rshift(Z_LIMB(tmp2), ptr_arg2 + limb2, size_arg2, bit2);
+        if (!Z_LIMB(tmp2)[size_arg2-1]) size_arg2--;
+      }
+      else ml_z_cpy_limb(Z_LIMB(tmp2), ptr_arg2 + limb2, size_arg2);
+      /* compute gcd of 2^pos1 & 2^pos2 */
+      pos = (pos1 <= pos2) ? pos1 : pos2;
+      limb = pos / Z_LIMB_BITS;
+      bit = pos % Z_LIMB_BITS;
+      /* compute gcd of arg1 & arg2 without lower 0 bits */
+      /* second argument must have less bits than first  */
+      if ((size_arg1 > size_arg2) ||
+          ((size_arg1 == size_arg2) && 
+           (Z_LIMB(tmp1)[size_arg1 - 1] >= Z_LIMB(tmp2)[size_arg1 - 1]))) {
+        r = ml_z_alloc(size_arg2 + limb + 1);
+        sz = mpn_gcd(Z_LIMB(r) + limb, Z_LIMB(tmp1), size_arg1, Z_LIMB(tmp2), size_arg2);
+      }
+      else {
+        r = ml_z_alloc(size_arg1 + limb + 1);
+        sz = mpn_gcd(Z_LIMB(r) + limb, Z_LIMB(tmp2), size_arg2, Z_LIMB(tmp1), size_arg1);
+      } 
+      /* glue the two results */
+      for (i = 0; i < limb; i++) 
+        Z_LIMB(r)[i] = 0;
+      Z_LIMB(r)[sz + limb] = 0;
+      if (bit) mpn_lshift(Z_LIMB(r) + limb, Z_LIMB(r) + limb, sz + 1, bit);
+      r = ml_z_reduce(r, limb + sz + 1, 0);
+    }
     Z_CHECK(r);
     CAMLreturn(r);
   }
@@ -2995,6 +2999,11 @@ struct custom_operations ml_z_custom_ops = {
   ml_z_custom_deserialize,
 #if Z_OCAML_COMPARE_EXT
   ml_z_custom_compare,
+#else
+  custom_compare_ext_default,
+#endif
+#ifndef Z_OCAML_LEGACY_CUSTOM_OPERATIONS
+  custom_fixed_length_default
 #endif
 };
 
