@@ -1,11 +1,11 @@
 (* Simple tests for the Z and Q modules.
 
 
-   This file is part of the Zarith library 
+   This file is part of the Zarith library
    http://forge.ocamlcore.org/projects/zarith .
    It is distributed under LGPL 2 licensing, with static linking exception.
    See the LICENSE file included in the distribution.
-   
+
    Copyright (c) 2010-2011 Antoine Miné, Abstraction project.
    Abstraction is part of the LIENS (Laboratoire d'Informatique de l'ENS),
    a joint laboratory by:
@@ -20,17 +20,30 @@
 
 module I = Z
 
-let pr ch x = 
-  output_string ch (I.to_string x); 
+let pr ch x =
+  output_string ch (I.to_string x);
   flush ch
 
-let pr2 ch (x,y) = 
-  Printf.fprintf ch "%s, %s" (I.to_string x) (I.to_string y); 
+let pr2 ch (x,y) =
+  Printf.fprintf ch "%s, %s" (I.to_string x) (I.to_string y);
   flush ch
 
-let pr3 ch (x,y,z) = 
-  Printf.fprintf ch "%s, %s, %s" 
-    (I.to_string x) (I.to_string y) (I.to_string z); 
+let pr3 ch (x,y,z) =
+  Printf.fprintf ch "%s, %s, %s"
+    (I.to_string x) (I.to_string y) (I.to_string z);
+  flush ch
+
+let prfloat ch (x,y : float * float) =
+  if x = y then
+    Printf.fprintf ch "OK"
+  else
+    Printf.fprintf ch "WRONG! (expected %g, got %g)" y x
+
+let prmarshal ch (x,y : I.t * I.t) =
+  (if I.equal x y then
+     Printf.fprintf ch "OK"
+   else
+     Printf.fprintf ch "WRONG! (expected %a, got %a)" pr y pr x);
   flush ch
 
 let pow2 n =
@@ -39,7 +52,7 @@ let pow2 n =
   in
   doit I.one n
 
-let fact n = 
+let fact n =
   let rec doit acc n =
   if n<=1 then acc
   else doit (I.mul acc (I.of_int n)) (n-1)
@@ -81,10 +94,9 @@ let chk_bits x =
   Printf.printf "\n";
   assert(I.equal (I.abs x) (I.of_bits (I.to_bits x)));
   assert((I.to_bits x) = (I.to_bits (I.neg x)));
-  Printf.printf "marshal %a\n =" pr x;
-  String.iter (fun c -> Printf.printf " %02x" (Char.code c)) (Marshal.to_string  x []);
-  Printf.printf "\n";
-  assert (x = Marshal.from_string (Marshal.to_string x []) 0)
+  Printf.printf "marshal round trip %a\n =" pr x;
+  let y = Marshal.(from_string (to_string x []) 0) in
+  Printf.printf " %a\n" prmarshal (y, x)
 
 let chk_extract (x, o, l) =
   let expected =
@@ -129,12 +141,12 @@ let chk_testbit x =
   let n = I.numbits x in
   let ok = ref true in
   for i = 0 to n + 64 do
-    let actual = I.testbit x i 
+    let actual = I.testbit x i
     and expected = I.extract x i 1 in
     if not (I.equal expected (if actual then I.one else I.zero))
     then begin Printf.printf "(error on %d) " i; ok := false end
   done;
-  if !ok 
+  if !ok
   then Printf.printf "(passed)\n"
   else Printf.printf "(FAILED)\n"
 
@@ -312,22 +324,22 @@ let test_Z() =
   Printf.printf "of_float -0.5\n = %a\n" pr (I.of_float (-. 0.5));
   Printf.printf "of_float 200.5\n = %a\n" pr (I.of_float 200.5);
   Printf.printf "of_float -200.5\n = %a\n" pr (I.of_float (-. 200.5));
-  Printf.printf "to_float 0\n = %f\n" (I.to_float I.zero);
-  Printf.printf "to_float 1\n = %f\n" (I.to_float I.one);
-  Printf.printf "to_float -1\n = %f\n" (I.to_float I.minus_one);
-  Printf.printf "to_float 2^120\n = %f\n" (I.to_float p120);
-  Printf.printf "to_float -2^120\n = %f\n" (I.to_float (I.neg p120));
-  Printf.printf "to_float (2^120-1)\n = %f\n" (I.to_float (I.pred p120));
-  Printf.printf "to_float (-2^120+1)\n = %f\n" (I.to_float (I.succ (I.neg p120)));
-  Printf.printf "to_float 2^63\n = %f\n" (I.to_float (pow2 63));
-  Printf.printf "to_float -2^63\n = %f\n" (I.to_float (I.neg (pow2 63)));
-  Printf.printf "to_float (2^63-1)\n = %f\n" (I.to_float (I.pred (pow2 63)));
-  Printf.printf "to_float (-2^63-1)\n = %f\n" (I.to_float (I.pred (I.neg (pow2 63))));
-  Printf.printf "to_float (-2^63+1)\n = %f\n" (I.to_float (I.succ (I.neg (pow2 63))));
-  Printf.printf "to_float 2^300\n = %f\n" (I.to_float p300);
-  Printf.printf "to_float -2^300\n = %f\n" (I.to_float (I.neg p300));
-  Printf.printf "to_float (2^300-1)\n = %f\n" (I.to_float (I.pred p300));
-  Printf.printf "to_float (-2^300+1)\n = %f\n" (I.to_float (I.succ (I.neg p300)));
+  Printf.printf "to_float 0\n = %a\n" prfloat (I.to_float I.zero, 0.0);
+  Printf.printf "to_float 1\n = %a\n" prfloat (I.to_float I.one, 1.0);
+  Printf.printf "to_float -1\n = %a\n" prfloat (I.to_float I.minus_one, -1.0);
+  Printf.printf "to_float 2^120\n = %a\n" prfloat (I.to_float p120, ldexp 1.0 120);
+  Printf.printf "to_float -2^120\n = %a\n" prfloat (I.to_float (I.neg p120), -. (ldexp 1.0 120));
+  Printf.printf "to_float (2^120-1)\n = %a\n" prfloat (I.to_float (I.pred p120), ldexp 1.0 120);
+  Printf.printf "to_float (-2^120+1)\n = %a\n" prfloat (I.to_float (I.succ (I.neg p120)), -. (ldexp 1.0 120));
+  Printf.printf "to_float 2^63\n = %a\n" prfloat (I.to_float (pow2 63), ldexp 1.0 63);
+  Printf.printf "to_float -2^63\n = %a\n" prfloat (I.to_float (I.neg (pow2 63)), -. (ldexp 1.0 63));
+  Printf.printf "to_float (2^63-1)\n = %a\n" prfloat (I.to_float (I.pred (pow2 63)), ldexp 1.0 63);
+  Printf.printf "to_float (-2^63-1)\n = %a\n" prfloat (I.to_float (I.pred (I.neg (pow2 63))), -. (ldexp 1.0 63));
+  Printf.printf "to_float (-2^63+1)\n = %a\n" prfloat (I.to_float (I.succ (I.neg (pow2 63))), -. (ldexp 1.0 63));
+  Printf.printf "to_float 2^300\n = %a\n" prfloat (I.to_float p300, ldexp 1.0 300);
+  Printf.printf "to_float -2^300\n = %a\n" prfloat (I.to_float (I.neg p300), -. (ldexp 1.0 300));
+  Printf.printf "to_float (2^300-1)\n = %a\n" prfloat (I.to_float (I.pred p300), ldexp 1.0 300);
+  Printf.printf "to_float (-2^300+1)\n = %a\n" prfloat (I.to_float (I.succ (I.neg p300)), -. (ldexp 1.0 300));
   Printf.printf "of_string 12\n = %a\n" pr (I.of_string "12");
   Printf.printf "of_string 0x12\n = %a\n" pr (I.of_string "0x12");
   Printf.printf "of_string 0b10\n = %a\n" pr (I.of_string "0b10");
@@ -493,12 +505,14 @@ let test_Z() =
   Printf.printf "sign -1\n = %i\n" (I.sign I.minus_one);
   Printf.printf "sign 2^300\n = %i\n" (I.sign p300);
   Printf.printf "sign -2^300\n = %i\n" (I.sign (I.neg p300));
+  Printf.printf "gcd 0 -137\n = %a\n" pr (I.gcd (I.of_int 0) (I.of_int (-137)));
   Printf.printf "gcd 12 27\n = %a\n" pr (I.gcd (I.of_int 12) (I.of_int 27));
   Printf.printf "gcd 27 12\n = %a\n" pr (I.gcd (I.of_int 27) (I.of_int 12));
   Printf.printf "gcd 27 27\n = %a\n" pr (I.gcd (I.of_int 27) (I.of_int 27));
   Printf.printf "gcd -12 27\n = %a\n" pr (I.gcd (I.of_int (-12)) (I.of_int 27));
   Printf.printf "gcd 12 -27\n = %a\n" pr (I.gcd (I.of_int 12) (I.of_int (-27)));
   Printf.printf "gcd -12 -27\n = %a\n" pr (I.gcd (I.of_int (-12)) (I.of_int (-27)));
+  Printf.printf "gcd 0 2^300\n = %a\n" pr (I.gcd (I.of_int 0) p300);
   Printf.printf "gcd 2^120 2^300\n = %a\n" pr (I.gcd p120 p300);
   Printf.printf "gcd 2^300 2^120\n = %a\n" pr (I.gcd p300 p120);
   Printf.printf "gcdext 12 27\n = %a\n" pr3 (I.gcdext (I.of_int 12) (I.of_int 27));
@@ -559,9 +573,9 @@ let test_Z() =
   Printf.printf "2^120 < 1\n = %B\n" (p120 < I.one);
   Printf.printf "-2^120 > 1\n = %B\n" ((I.neg p120) > I.one);
   Printf.printf "-2^120 < 1\n = %B\n" ((I.neg p120) < I.one);
-  Printf.printf "demarshal 2^120, 2^300, 1\n = %a\n" pr3 
+  Printf.printf "demarshal 2^120, 2^300, 1\n = %a\n" pr3
     (Marshal.from_string (Marshal.to_string (p120,p300,I.one) []) 0);
-  Printf.printf "demarshal -2^120, -2^300, -1\n = %a\n" pr3 
+  Printf.printf "demarshal -2^120, -2^300, -1\n = %a\n" pr3
     (Marshal.from_string (Marshal.to_string (I.neg p120,I.neg p300,I.minus_one) []) 0);
   Printf.printf "format %%i 0 = /%s/\n" (I.format "%i" I.zero);
   Printf.printf "format %%i 1 = /%s/\n" (I.format "%i" I.one);
@@ -690,13 +704,13 @@ let test_Z() =
   chk_bits minni;
 
   List.iter chk_testbit [
-    I.zero; I.one; I.of_int (-42); 
+    I.zero; I.one; I.of_int (-42);
     I.of_string "31415926535897932384626433832795028841971693993751058209749445923078164062862089986";
     I.neg (I.shift_left (I.of_int 123456) 64);
   ];
 
   List.iter chk_numbits_tz [
-    I.zero; I.one; I.of_int (-42); 
+    I.zero; I.one; I.of_int (-42);
     I.shift_left (I.of_int 9999) 77;
     I.neg (I.shift_left (I.of_int 123456) 64);
   ];
@@ -721,7 +735,7 @@ let t_list = [Q.zero;Q.one;Q.minus_one;Q.inf;Q.minus_inf;Q.undef]
 
 let test1 msg op =
   List.iter
-    (fun x -> 
+    (fun x ->
       let r = op x in
       check r;
       Printf.printf "%s %s = %s\n" msg (Q.to_string x)  (Q.to_string r)
@@ -729,9 +743,9 @@ let test1 msg op =
 
 let test2 msg op =
   List.iter
-    (fun x -> 
+    (fun x ->
       List.iter
-        (fun y -> 
+        (fun y ->
           let r = op x y in
           check r;
           Printf.printf "%s %s %s = %s\n" (Q.to_string x) msg (Q.to_string y) (Q.to_string r)
@@ -743,7 +757,7 @@ let test_Q () =
   let _ = test1 "-" Q.neg in
   let _ = test1 "1/" Q.inv in
   let _ = test1 "abs" Q.abs in
-  let _ = test2 "+" Q.add in 
+  let _ = test2 "+" Q.add in
   let _ = test2 "-" Q.sub in
   let _ = test2 "*" Q.mul in
   let _ = test2 "/" Q.div in
@@ -754,11 +768,11 @@ let test_Q () =
   let _ = test1 "div_2exp (2) " (fun a -> Q.div_2exp a 2) in
   (* check simple identitites *)
   List.iter
-    (fun x -> 
+    (fun x ->
       assert (Q.equal x (Q.div_2exp (Q.mul_2exp x 2) 2));
       assert (Q.equal x (Q.mul_2exp (Q.div_2exp x 2) 2));
       List.iter
-        (fun y -> 
+        (fun y ->
           Printf.printf "identity checking %s %s\n" (Q.to_string x) (Q.to_string y);
           assert (Q.equal (Q.add x y) (Q.add y x));
           assert (Q.equal (Q.sub x y) (Q.neg (Q.sub y x)));
@@ -773,8 +787,3 @@ let test_Q () =
 
 let _ = test_Z()
 let _ = test_Q()
-
-
-
-
-
